-- ищет межфакультетские дисциплины по семестру, ведущему факультету, предмету и типу
-- межфакультетскими считает все дисциплины, которые не связаны с учебными планами
DROP FUNCTION IF EXISTS public.discipline_find_global(psemesterid integer, psubjectid integer, pexamtype exam_credit_grading_credit, pfacultyid integer);
CREATE OR REPLACE FUNCTION public.discipline_find_global(psemesterid integer, psubjectid integer, pexamtype exam_credit_grading_credit, pfacultyid integer)
    RETURNS SETOF integer
    LANGUAGE sql
AS $function$
    --select public.discipline_find_by_subject(
--	:psemesterid,	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--	:psubjectid,	-- put the psubjectid parameter value instead of 'psubjectid' (int4)
--	:pexamtype 	-- put the pexamtype parameter value instead of 'pexamtype' (exam_credit_grading_credit)
--);
SELECT disciplines.ID
FROM disciplines
WHERE disciplines.SemesterID = pSemesterID AND
        disciplines.SubjectID = pSubjectID AND
        disciplines.ExamType = pExamType AND
        disciplines.FacultyID = pfacultyid AND
        NOT EXISTS (SELECT disciplines_study_plans.id from disciplines_study_plans WHERE disciplines_study_plans.DisciplineID = disciplines.ID)
LIMIT 1;
$function$;

DROP FUNCTION IF EXISTS public.internalbindstudent(pdisciplineid integer, precordbookid integer);
CREATE OR REPLACE FUNCTION public.internalbindstudent(pdisciplineid integer, precordbookid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    -- bind  student
    INSERT INTO disciplines_students (DisciplineID, recordbookid, "type") VALUES (pDisciplineID, precordbookid, 'attach')
    ON conflict ON CONSTRAINT disciplines_students_recordbookid_disciplineid_key
        DO UPDATE set "type" = 'attach';
   return 1;
   EXCEPTION 
    when others then RETURN -1;
END
$function$;

DROP FUNCTION IF EXISTS public.internalunbindstudent(pdisciplineid integer, precordbookid integer);
CREATE OR REPLACE FUNCTION public.internalunbindstudent(pdisciplineid integer, precordbookid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    -- unbind  student
    update disciplines_students 
    set "type" = 'detach'
    where DisciplineID=pdisciplineid and recordbookid=precordbookid;
    return 1;
   EXCEPTION 
    when others then RETURN -1;
END
$function$;

DROP FUNCTION IF EXISTS public.discipline_getinfo(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.discipline_getinfo(pdisciplineid integer)
    RETURNS TABLE("ID" integer, "AuthorID" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "Type" exam_credit_grading_credit, "Lectures" integer, "Practice" integer, "Labs" integer, "SemesterID" integer, "SubjectID" integer, "SubjectName" character varying, "SubjectAbbr" character varying, "FacultyID" integer, "FacultyName" character varying, "IsLocked" integer, "Milestone" integer, "Subtype" scientific_disciplinary_coursework, "CompoundDiscID" integer, "IsMapCreated" boolean, "IsGlobal" boolean, "IsBonus" boolean, "semesterNum" integer, "semesterYear" integer)
    LANGUAGE plpgsql
AS $function$
DECLARE vIsBonus BOOLEAN;
begin
    --	select * from public.discipline_getinfo(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    vIsBonus := EXISTS(
            SELECT * FROM modules
            WHERE   modules.DisciplineID = pDisciplineID AND
                    modules.Type = 'bonus'
            LIMIT 1
        );
    return query
        SELECT
            view_disciplines.DisciplineID AS "ID",
            view_disciplines.AuthorID as "AuthorID",
            view_disciplines.GradeID as "GradeID",
            view_disciplines.GradeNum as "GradeNum",
            view_disciplines."degree" as "Degree",
            view_disciplines.ExamType       AS "Type",
            view_disciplines.LectureCount   AS "Lectures",
            view_disciplines.PracticeCount  AS "Practice",
            view_disciplines.LabCount       AS "Labs",
            view_disciplines.SemesterID as "SemesterID",
            view_disciplines.SubjectID as "SubjectID",
            view_disciplines.SubjectName as "SubjectName",
            view_disciplines.SubjectAbbr as "SubjectAbbr",
            view_disciplines.FacultyID as "FacultyID",
            view_disciplines.FacultyName as "FacultyName",
            view_disciplines.IsLocked as "IsLocked",
            view_disciplines.Milestone as "Milestone",
            view_disciplines.Subtype as "Subtype",
            view_disciplines.CompoundDiscID as "CompoundDiscID",
            (view_disciplines.MaxRate = 100) AS "IsMapCreated",
            NOT EXISTS (SELECT disciplines_study_plans.id from disciplines_study_plans WHERE disciplines_study_plans.DisciplineID = view_disciplines.DisciplineID) AS "IsGlobal",
            vIsBonus AS "IsBonus",
            semesters.Num AS "semesterNum", -- TODO: Camelize
            semesters.Year AS "semesterYear"
        FROM view_disciplines
                 INNER JOIN semesters ON semesters.ID = view_disciplines.SemesterID
        WHERE view_disciplines.DisciplineID = pDisciplineID
        LIMIT 1;
END ;
$function$;

DROP FUNCTION IF EXISTS public.getdisciplinesforteacher(pteacherid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.getdisciplinesforteacher(pteacherid integer, psemesterid integer)
    RETURNS TABLE("ID" integer, "Type" exam_credit_grading_credit, "SubType" scientific_disciplinary_coursework, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer, "GroupName" character varying, "SubjectID" integer, "SubjectName" character varying, "AuthorID" integer, "IsLocked" integer, "IsMapCreated" boolean, "IsGlobal" boolean)
    LANGUAGE sql
AS $function$
    --select public.getdisciplinesforteacher(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
SELECT DISTINCT view_disciplines.DisciplineID AS "ID",
                view_disciplines.ExamType AS "Type",
                view_disciplines.Subtype as "SubType",
                view_disciplines.GradeID as "GradeID",
                view_disciplines.GradeNum as "GradeNum",
                view_disciplines."degree" as "Degree",
                view_groups.GroupID as "GroupID",
                view_groups.GroupNum as "GroupNum",
                view_groups.GroupName as "GroupName",
                view_disciplines.SubjectID as "SubjectID",
                view_disciplines.SubjectName as "SubjectName",
                view_disciplines.AuthorID as "AuthorID",
                view_disciplines.IsLocked AS "IsLocked", -- bodging, db schema remembered lowerCase
                (view_disciplines.MaxRate = 100)    AS "IsMapCreated",
                NOT EXISTS (SELECT disciplines_study_plans.id from disciplines_study_plans WHERE disciplines_study_plans.DisciplineID = view_disciplines.DisciplineID) AS "IsGlobal"
FROM disciplines_teachers
         INNER JOIN view_disciplines ON disciplines_teachers.DisciplineID = view_disciplines.DisciplineID
         INNER JOIN semesters ON semesters.ID = view_disciplines.SemesterID
         LEFT JOIN disciplines_groups ON disciplines_groups.DisciplineID = disciplines_teachers.DisciplineID
         LEFT JOIN view_groups ON view_groups.GroupID = disciplines_groups.GroupID AND view_groups.Year = semesters.Year
WHERE disciplines_teachers.TeacherID = pTeacherID AND view_disciplines.SemesterID = pSemesterID
ORDER BY    view_disciplines.GradeID ASC,
            view_disciplines.SubjectName ASC,
            view_disciplines.DisciplineID ASC,
            view_groups.GroupNum ASC;
$function$;

DROP FUNCTION IF EXISTS public.getfinalforminfoglobal(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.getfinalforminfoglobal(pdisciplineid integer)
    RETURNS TABLE("FacultyID" integer, "FacultyName" character varying, "FacultyAbbr" character varying, "ExamType" exam_credit_grading_credit, "SubjectID" integer, "SubjectName" character varying, "SubjectAbbr" character varying, "AuthorID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "JobPosition" character varying, "Year" integer, "SemesterNum" integer)
    LANGUAGE sql
AS $function$
SELECT  faculties.ID AS "FacultyID",
        faculties.Name AS "FacultyName",
        faculties.Abbr AS "FacultyAbbr",
        disciplines.ExamType AS "ExamType",
        subjects.ID AS "SubjectID",
        subjects.Name AS "SubjectName",
        subjects.Abbr AS "SubjectAbbr",
        teachers.ID AS "AuthorID",
        accounts.LastName AS "LastName",
        accounts.FirstName AS "FirstName",
        accounts.SecondName AS "SecondName",
        job_positions.Name AS "JobPosition",
        semesters."year" AS "Year",
        semesters.Num AS "SemesterNum"
FROM disciplines
         INNER JOIN subjects ON disciplines.SubjectID = subjects.ID
         INNER JOIN teachers ON teachers.ID = disciplines.AuthorID
         INNER JOIN accounts ON teachers.AccountID = accounts.ID
         INNER JOIN job_positions ON job_positions.ID = teachers.JobPositionID
         INNER JOIN semesters ON disciplines.SemesterID = semesters.ID
         INNER JOIN faculties ON faculties.ID = disciplines.FacultyID
WHERE disciplines.ID = pdisciplineid
LIMIT 1;
$function$;

DROP FUNCTION IF EXISTS public.getratesforstudentsbystage(pdisciplineid integer, pmilestone integer);
CREATE OR REPLACE FUNCTION public.getratesforstudentsbystage(pdisciplineid integer, pmilestone integer)
    RETURNS TABLE("StudentID" integer, "Semester" integer, "Bonus" integer, "Extra" integer, "PreviousExtra" integer, "Exam" integer, "PreviousExam" integer, "Option" absence_pass, "AutoPassed" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying)
    LANGUAGE plpgsql
AS $function$

DECLARE vSemesterID INT DEFAULT -1;
        vInGeneralGroup BOOL DEFAULT FALSE;
BEGIN
    vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');

    DROP TABLE IF EXISTS tStudents;
    CREATE TEMPORARY TABLE tStudents AS (
        SELECT record_books.ID, record_books.StudentID
        FROM record_books
                 LEFT
                     JOIN disciplines_students on disciplines_students.DisciplineID = pDisciplineid and
                                                  disciplines_students.RecordBookID = record_books.ID
                 left join view_disciplines_recordbooks on disciplines_students.recordbookid = view_disciplines_recordbooks.recordbookid
        WHERE   COALESCE(disciplines_students.Type, 'attach') = 'attach' and
                view_disciplines_recordbooks.disciplineid = pDisciplineid
    );

    return query
        SELECT  tRes.*,
                accounts.LastName as "LastName",
                accounts.FirstName as "FirstName",
                accounts.SecondName as "SecondName"
        FROM (
                 SELECT  tStudents.StudentID as "StudentID",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'regular',1,0)) as integer) AS "Semester",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'bonus',1,0)) as integer) AS "Bonus",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'extra',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone,1,0)) as integer) AS "Extra",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'extra',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone - 1,1,0)) as integer) AS "PreviousExtra",
                         cast (SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'exam',1,0)*iif_sql(tMap.SubmoduleOrderNum = pMilestone,1,0)) as integer) AS "Exam",
                         MAX(tRate.Rate*iif_sql(tMap.ModuleType = 'exam',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone,1,0)) AS "PreviousExam",
                         MAX(CASE WHEN tMap.SubmoduleOrderNum = pMilestone THEN  exam_period_options."type" ELSE  NULL END) As "Option",
                         MAX(CASE WHEN exam_period_options."type" = 'pass' THEN  1 ELSE  0 END) As "AutoPassed"
                 FROM tStudents
                          LEFT JOIN view_roadmap AS tMap   ON  tMap.DisciplineID = pDisciplineID
                          LEFT JOIN rating_table AS tRate  ON  tRate.RecordbookID = tStudents.ID AND
                                                               tRate.SubmoduleID = tMap.SubmoduleID
                          LEFT JOIN exam_period_options ON  exam_period_options.submoduleID = tMap.SubmoduleID AND
                                                            exam_period_options.RecordbookID = tStudents.ID
                 GROUP BY tStudents.StudentID
             ) tRes
                 INNER JOIN students ON students.ID = tRes."StudentID"
                 INNER JOIN accounts ON students.AccountID = accounts.ID
        ORDER BY    accounts.LastName ASC,
                    accounts.FirstName ASC,
                    accounts.SecondName ASC;
END
$function$;