CREATE OR REPLACE FUNCTION public.finalreport_getbysemesteryearnum(pyear integer, pnum integer)
  RETURNS TABLE("SemesterYear" integer, "SemesterNum" integer, "PlanExternalID" integer,
                "SubjectExternalID" character varying, "RecordBookExternalID" character varying,
                "SemesterRate" bigint, "ExamRate" integer, "Exam2Rate" integer, "Exam3Rate" integer,
                "ExtraRate" integer, "Extra2Rate" integer)
LANGUAGE plpgsql
AS $function$
BEGIN
  return query
  SELECT *
  FROM view_final_report
  WHERE view_final_report.SemesterYear = pyear and
        view_final_report.SemesterNum = pnum;
END
$function$;

DROP FUNCTION IF EXISTS public.finalreport_getbysemesteryearplan(pyear integer, pnum integer, pplanexternalid integer);
CREATE OR REPLACE FUNCTION public.finalreport_getbysemesteryearplan(pyear integer, pnum integer, pplanexternalid integer)
  RETURNS TABLE("SemesterYear" integer, "SemesterNum" integer, "PlanExternalID" integer,
                "SubjectExternalID" character varying, "RecordBookExternalID" character varying,  "ExamType" exam_credit_grading_credit,
                "SemesterRate" bigint, "ExamRate" integer, "Exam2Rate" integer, "Exam3Rate" integer,
                "ExtraRate" integer, "Extra2Rate" integer)
LANGUAGE plpgsql
AS $function$
BEGIN
  return query
  SELECT *
  FROM view_final_report
  WHERE view_final_report.SemesterYear = pyear and
        view_final_report.SemesterNum = pnum and
        view_final_report.PlanExternalID = pplanexternalid;
END
$function$;

DROP FUNCTION IF EXISTS public.finalreport_getbysemesteryeardiscipline(pyear integer, pnum integer, pdisciplineexternalid character varying);
CREATE OR REPLACE FUNCTION public.finalreport_getbysemesteryeardiscipline(pyear integer, pnum integer, pdisciplineexternalid character varying)
  RETURNS TABLE("SemesterYear" integer, "SemesterNum" integer, "PlanExternalID" integer,
                "SubjectExternalID" character varying, "RecordBookExternalID" character varying, "ExamType" exam_credit_grading_credit,
                "SemesterRate" bigint, "ExamRate" integer, "Exam2Rate" integer, "Exam3Rate" integer,
                "ExtraRate" integer, "Extra2Rate" integer)
LANGUAGE plpgsql
AS $function$
BEGIN
  return query
  SELECT *
  FROM view_final_report
  WHERE view_final_report.SemesterYear = pyear and
        view_final_report.SemesterNum = pnum and
        view_final_report.SubjectExternalID = pdisciplineexternalid;
END
$function$;


DROP FUNCTION IF EXISTS public.finalreport_getbysemesteryeardisciplinegroup(pyear integer, pnum integer, pdisciplineexternalid character varying, pgroupid integer);
CREATE OR REPLACE FUNCTION public.finalreport_getbysemesteryeardisciplinegroup(pyear integer, pnum integer, pdisciplineexternalid character varying, pgroupid integer)
  RETURNS TABLE("SemesterYear" integer, "SemesterNum" integer, "PlanExternalID" integer,
                "SubjectExternalID" character varying, "RecordBookExternalID" character varying, "ExamType" exam_credit_grading_credit,
                "SemesterRate" bigint, "ExamRate" integer, "Exam2Rate" integer, "Exam3Rate" integer,
                "ExtraRate" integer, "Extra2Rate" integer)
LANGUAGE plpgsql
AS $function$
declare vSemesterId integer default -1;
BEGIN

  select semesters.id into vSemesterId
  from semesters
  where semesters.year = pyear and semesters.num = pnum;

  return query
  select DisciplineGrades.SemesterYear as "SemesterYear",
         DisciplineGrades.SemesterNum as "SemesterNum",
         DisciplineGrades.PlanExternalID as "PlanExternalDI",
         DisciplineGrades.SubjectExternalID as "SubjectExternalID",
         DisciplineGrades.RecordBookExternalID as "RecordBookExternalID",
         DisciplineGrades.ExamType as "ExamType",
         DisciplineGrades.SemesterRate as "SemesterRate",
         DisciplineGrades.ExamRate as "ExamRate",
         DisciplineGrades.Exam2Rate as "Exam2Rate",
         DisciplineGrades.Exam3Rate as "Exam3Rate",
         DisciplineGrades.ExtraRate as "ExtraRate",
         DisciplineGrades.Extra2Rate as "Extra2Rate"
  from
    (SELECT *
     FROM view_final_report
     WHERE view_final_report.SemesterYear = pyear and
           view_final_report.SemesterNum = pnum and
           view_final_report.SubjectExternalID = pdisciplineexternalid) as DisciplineGrades
    join record_books on record_books.externalid = DisciplineGrades.recordbookexternalid
    join students_groups on students_groups.recordbookid = record_books.id
  where students_groups.semesterid = vSemesterId and students_groups.groupid = pgroupid;
END
$function$;

drop function if exists public.findgroup(pgradeid integer, pgroupnum integer, pfacultyid integer, pformid integer);
CREATE OR REPLACE FUNCTION public.findgroup(pgradeid integer, pgroupnum integer, pfacultyid integer, pformid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
begin

  RETURN COALESCE((
                    SELECT study_groups.ID FROM study_groups
                    WHERE   study_groups.GradeID = pGradeID
                            AND study_groups.GroupNum = pGroupNum
                            AND study_groups.FacultyID = pFacultyID
                    --AND (study_groups.formid = pFormID
                    --OR study_groups.formid = null)
                    LIMIT 1
                  ), -1);
END
$function$
;

CREATE OR REPLACE VIEW public.view_final_report
  AS SELECT DISTINCT semesters.year AS semesteryear,
                     semesters.num AS semesternum,
                     study_plans.externalid AS planexternalid,
                     subjects.externalid AS subjectexternalid,
                     record_books.externalid AS recordbookexternalid,
       disciplines.examtype,
                     ( SELECT sum(rating_table.rate) AS sum
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'regular'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       GROUP BY modules.disciplineid, rating_table.recordbookid) AS semesterrate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'exam'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       LIMIT 1) AS examrate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'exam'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       OFFSET 1
                       LIMIT 1) AS exam2rate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'exam'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       OFFSET 2
                       LIMIT 1) AS exam3rate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'extra'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       LIMIT 1) AS extrarate,
                     ( SELECT rating_table.rate
                       FROM rating_table
                         JOIN submodules ON rating_table.submoduleid = submodules.id
                         JOIN modules ON submodules.moduleid = modules.id
                       WHERE modules.disciplineid = disciplines.id AND modules.type = 'extra'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                       ORDER BY submodules.ordernum
                       OFFSET 1
                       LIMIT 1) AS extra2rate
     FROM semesters
       JOIN disciplines ON disciplines.semesterid = semesters.id
       JOIN disciplines_study_plans ON disciplines_study_plans.disciplineid = disciplines.id
       JOIN study_plans ON study_plans.id = disciplines_study_plans.studyplanid
       JOIN subjects ON subjects.id = disciplines.subjectid
       JOIN view_disciplines_recordbooks ON view_disciplines_recordbooks.disciplineid = disciplines.id AND 'attach'::attach_detach = COALESCE(view_disciplines_recordbooks.type, 'attach'::attach_detach)
       JOIN record_books ON record_books.id = view_disciplines_recordbooks.recordbookid
       inner join record_books_plans on record_books.id = record_books_plans.recordbookid and record_books_plans.studyplanid = study_plans.id;

drop function if exists public.getdisciplinesforexport(pgroupid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.getdisciplinesforexport(pgroupid integer, psemesterid integer)
  RETURNS TABLE("ID" integer, "SubjectName" character varying, "DisciplineExternalID" character varying,
                "PlanExternalID" integer)
LANGUAGE plpgsql
AS $function$
BEGIN
  return query
  select D."ID" as "ID",
         D."SubjectName" as "SubjectName",
         subjects.externalid as "DisciplineExternalID",
         study_plans.externalid as "PlanExternalID"
  from public.getdisciplinesforgroup(pgroupid, psemesterid) as D
    join disciplines on disciplines.id = D."ID"
    join subjects on disciplines.subjectid = subjects.id
    join disciplines_study_plans on disciplines_study_plans.disciplineid = D."ID"
    join students_groups on public.students_groups.groupid = pgroupid
    join record_books_plans on record_books_plans.recordbookid = public.students_groups.recordbookid
    join study_plans on disciplines_study_plans.studyplanid = study_plans.id and record_books_plans.studyplanid = study_plans.id;

end;
$function$;


CREATE SEQUENCE seq_logs_form_export
  START WITH 1
  INCREMENT BY 1
  NO MINVALUE
  NO MAXVALUE
  CACHE 1;

DROP TABLE IF EXISTS logs_form_export;
CREATE TABLE logs_form_export (
  id integer DEFAULT nextval('seq_logs_form_export'::regclass) NOT NULL,
  discipline_id integer NOT NULL,
  group_id integer NOT NULL,
  account_id integer NOT NULL,
  semester_id integer NOT NULL,
  error integer NOT NULL DEFAULT 0,
  date timestamp(0) without time zone DEFAULT now() NOT NULL
);

CREATE INDEX key_logs_form_export ON logs_form_export USING btree (discipline_id);

ALTER TABLE ONLY logs_form_export
  ADD CONSTRAINT logs_form_export_pkey PRIMARY KEY (id);

ALTER TABLE ONLY logs_form_export
  ADD CONSTRAINT logs_form_export_ibfk_1 FOREIGN KEY (discipline_id) REFERENCES disciplines(id);

ALTER TABLE ONLY logs_form_export
  ADD CONSTRAINT logs_form_export_ibfk_2 FOREIGN KEY (group_id) REFERENCES study_groups(id);

ALTER TABLE ONLY logs_form_export
  ADD CONSTRAINT logs_form_export_ibfk_3 FOREIGN KEY (account_id) REFERENCES accounts(id);

ALTER TABLE ONLY logs_form_export
  ADD CONSTRAINT logs_form_export_ibfk_4 FOREIGN KEY (semester_id) REFERENCES semesters(id);

DROP FUNCTION IF EXISTS public.LogFormExport(paccountid integer, pdisciplineid character varying, pgroupid integer, psemesterid integer, perror integer);
CREATE OR REPLACE FUNCTION public.LogFormExport(paccountid integer, pdisciplineid character varying, pgroupid integer, psemesterid integer, perror integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vDisciplineID integer;
begin
  select disciplinesForGroup."ID" into vDisciplineID from
    (select D."ID" as "ID",
            D."SubjectName" as "SubjectName",
            subjects.externalid as "DisciplineExternalID"
     from public.getdisciplinesforgroup(pgroupid, psemesterid) as D
       join disciplines on disciplines.id = D."ID"
       join subjects on disciplines.subjectid = subjects.id) as disciplinesForGroup
  WHERE disciplinesForGroup."DisciplineExternalID" = pdisciplineid;

  INSERT INTO logs_form_export (discipline_id, group_id, account_id, semester_id, error) VALUES (vDisciplineID, pgroupid, paccountid, psemesterid, perror);
  RETURN 0;
END
$function$;

drop function if exists public.GetFormExportDate(pdisciplineid integer, pgroupid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.GetFormExportDate(pdisciplineid integer, pgroupid integer, psemesterid integer)
  RETURNS DATE
LANGUAGE plpgsql
AS $function$
DECLARE vLastDate DATE DEFAULT NULL;
begin
  SELECT logs_form_export.date INTO vLastDate
  FROM logs_form_export
  WHERE logs_form_export.discipline_id = pdisciplineid
        AND logs_form_export.group_id = pgroupid
        AND logs_form_export.semester_id = psemesterid
  ORDER BY logs_form_export.date DESC
  LIMIT 1;
  return vLastDate;
END
$function$;

drop function if exists public.SetLastFormExportError(pdisciplineid integer, pgroupid integer, psemesterid integer, perror integer);
CREATE OR REPLACE FUNCTION public.SetLastFormExportError(pdisciplineid integer, pgroupid integer, psemesterid integer, perror integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vUpdateCount integer DEFAULT 0;
begin
  UPDATE logs_form_export
  SET error = perror
  WHERE logs_form_export.id = (
    SELECT logs_form_export.id FROM logs_form_export
    WHERE logs_form_export.discipline_id = pdisciplineid
          AND logs_form_export.group_id = pgroupid
          AND logs_form_export.semester_id = psemesterid
    ORDER BY logs_form_export.date DESC
    LIMIT 1
  );
  get diagnostics vUpdateCount = ROW_COUNT;
  return vUpdateCount;
END
$function$;


drop function if exists public.IsFormExportFailed(pdisciplineid integer, pgroupid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.IsFormExportFailed(pdisciplineid integer, pgroupid integer, psemesterid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vLastError integer DEFAULT 0;
begin
  SELECT logs_form_export.error INTO vLastError
  FROM logs_form_export
  WHERE logs_form_export.discipline_id = pdisciplineid
        AND logs_form_export.group_id = pgroupid
        AND logs_form_export.semester_id = psemesterid
  ORDER BY logs_form_export.date DESC
  LIMIT 1;
  return vLastError;
END
$function$;

drop function if exists public.IsFormOutdated(pdisciplineid integer, pgroupid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.IsFormOutdated(pdisciplineid integer, pgroupid integer, psemesterid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vLastDate DATE DEFAULT NULL;
        vCountNewGrades integer DEFAULT 0;
begin
  SELECT logs_form_export.date INTO vLastDate
  FROM logs_form_export
  WHERE logs_form_export.discipline_id = pdisciplineid
        AND logs_form_export.group_id = pgroupid
        AND logs_form_export.semester_id = psemesterid
  ORDER BY logs_form_export.date DESC
  LIMIT 1;

  SELECT count(logs_rating.date) INTO vCountNewGrades FROM
    logs_rating JOIN students_groups
      ON logs_rating.recordbookid = students_groups.recordbookid
    JOIN submodules ON logs_rating.submoduleid = submodules.id
    JOIN modules ON submodules.moduleid = modules.id
    JOIN disciplines ON modules.disciplineid = disciplines.id
  WHERE students_groups.semesterid = psemesterid
        AND students_groups.groupid = pgroupid
        AND disciplines.id = pdisciplineid
        AND logs_rating.date > vLastDate;

  return vCountNewGrades;
END
$function$;

update study_groups set formid=1;

ALTER TABLE public.study_groups ALTER COLUMN formid SET NOT NULL;
ALTER TABLE public.study_groups ALTER COLUMN formid SET DEFAULT 1;

ALTER TABLE public.study_groups DROP CONSTRAINT study_groups_facultyid_gradeid_groupnum_key;
ALTER TABLE ONLY public.study_groups
  ADD CONSTRAINT study_groups_facultyid_gradeid_groupnum_formid_key UNIQUE (facultyid, gradeid, groupnum, formid);

INSERT INTO public.study_form(id, formname) VALUES
  (1, 'очная'),
  (2, 'очно-заочная'),
  (3, 'заочная');

ALTER TABLE ONLY public.study_groups
  ADD CONSTRAINT study_groups_ibfk_4 FOREIGN KEY (formid) REFERENCES study_form(id);

DROP FUNCTION IF EXISTS public.creategroup(pgradeid integer, pgroupnum integer, pspecname character varying, pfacultyid integer, pyear integer);
CREATE OR REPLACE FUNCTION public.creategroup(pgradeid integer, pgroupnum integer, pspecname character varying, pfacultyid integer, pyear integer, pformid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vGroupID INT DEFAULT null;
        vSpecId INT DEFAULT null;
        vGroupYear INT DEFAULT null;
        vIsSpecMatch BOOL DEFAULT null;

BEGIN
  --select public.creategroup(
  --	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
  --	:pgroupnum,	-- put the pgroupnum parameter value instead of 'pgroupnum' (int4)
  --	:pspecname,	-- put the pspecname parameter value instead of 'pspecname' (varchar)
  --	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
  --	:pyear 	-- put the pyear parameter value instead of 'pyear' (int4)
  --);
  -- create specialization vSpecId - UNIQUE (pSpecName,pFacultyID)
  select specializations.id into vSpecId
  from specializations
  where specializations.facultyid=pFacultyID and specializations."name" like pSpecName;
  if (vSpecId is null) then
    INSERT INTO specializations (Name, Abbr, FacultyID)
    VALUES  (pSpecName, NULL, pFacultyID)
    returning id into vSpecId;
  end if;


  -- create group  vGroupID - UNIQUE (pGradeID, pGroupNum, pFacultyID)
  select study_groups.id into vGroupID
  from study_groups
  where study_groups.gradeid=pgradeID
        and study_groups.groupnum= pGroupNum
        and study_groups.facultyid=pFacultyID
        and study_groups.formid=pFormID;
  if ( vGroupID  is null) then
    INSERT INTO study_groups (GradeID, GroupNum, FacultyID, FormID)
    VALUES (pGradeID, pGroupNum, pFacultyID, pFormID)
    returning id into  vGroupID ;
  end if;


  SELECT groups_years.groupid, groups_years.specializationid = vSpecId
  INTO vGroupYear, vIsSpecMatch
  FROM groups_years
  WHERE groups_years.groupid = vGroupID AND groups_years."year" = pYear
  LIMIT 1;

  IF  (vGroupYear is null) THEN
    INSERT INTO groups_years (GroupID, Year, SpecializationID)
    VALUES (vGroupID, pYear, vSpecId);
  ELSEIF NOT vIsSpecMatch THEN
    RETURN -1;
  END IF;

  RETURN vGroupID;
  EXCEPTION
  when others then
    RETURN -1;
END
$function$;

DROP FUNCTION IF EXISTS public.getgroupsfordisciplineall(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.getgroupsfordisciplineall(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "GroupNum" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying, "FormID" character varying)
LANGUAGE plpgsql
AS $function$

DECLARE vSemesterID INT DEFAULT -1; vYear INT DEFAULT -1;
BEGIN
  vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');
  SELECT "year"  into vYear from semesters where semesters.ID = vSemesterID;

  -- general + attached
  CREATE TEMPORARY TABLE IF NOT EXISTS tGroup AS (
    SELECT students_groups.GroupID
    FROM view_disciplines_recordbooks
      INNER JOIN students_groups ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID AND
                                    students_groups.SemesterID = vSemesterID AND
                                    students_groups.State <= 'outlet'
    WHERE view_disciplines_recordbooks.DisciplineID = pDisciplineID AND
          'attach' = COALESCE(view_disciplines_recordbooks.Type, 'attach')
    GROUP BY students_groups.GroupID
  );

  return query
  SELECT  view_groups.GroupID AS "ID",
          view_groups.GroupNum as "GroupNum",
          view_groups.GradeID	as "GradeID",
          view_groups.GradeNum as "GradeNum",
          view_groups."degree" as "Degree",
          view_groups.SpecID as "SpecID",
          view_groups.SpecName as "SpecName",
          view_groups.SpecAbbr as "SpecAbbr",
    (SELECT study_form.FormName from study_form WHERE study_form.id = view_groups.FormID)
  --view_groups.FormID as "FormID"
  FROM tGroup
    INNER JOIN view_groups ON tGroup.GroupID = view_groups.GroupID AND view_groups.Year = vYear
  ORDER BY view_groups.GradeID ASC, view_groups.GroupID ASC;
END;
$function$;

DROP VIEW public.view_groups;
CREATE OR REPLACE VIEW public.view_groups
  AS SELECT study_groups.id AS groupid,
       study_groups.groupnum,
            groups_years.name AS groupname,
            grades.id AS gradeid,
            grades.num AS gradenum,
       grades.degree,
            specializations.id AS specid,
            specializations.name AS specname,
            specializations.abbr AS specabbr,
            specializations.code AS speccode,
            study_groups.formid AS formid,
            faculties.id AS facultyid,
            faculties.name AS facultyname,
            faculties.abbr AS facultyabbr,
       groups_years.year
     FROM groups_years
       JOIN study_groups ON groups_years.groupid = study_groups.id
       JOIN specializations ON groups_years.specializationid = specializations.id
       JOIN grades ON study_groups.gradeid = grades.id
       JOIN faculties ON faculties.id = specializations.facultyid;

DROP FUNCTION IF EXISTS public.getgroups(pgradeid integer, pfacultyid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.getgroups(pgradeid integer, pfacultyid integer, psemesterid integer)
  RETURNS TABLE("ID" integer, "GroupNum" integer, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying, "FormID" character varying)
LANGUAGE sql
AS $function$
--select * from public.getgroups(
--	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
SELECT  view_groups.GroupID AS "ID",
        view_groups.GroupNum as "GroupNum",
        view_groups.SpecID as "SpecID",
        view_groups.SpecName as "SpecName",
        view_groups.SpecAbbr as "SpecAbbr",
  (SELECT study_form.FormName from study_form WHERE study_form.id = view_groups.FormID)
FROM view_groups
  INNER JOIN semesters ON semesters.ID = pSemesterID
WHERE   view_groups.GradeID = pGradeID AND
        view_groups.FacultyID = pFacultyID AND
        view_groups.Year = semesters.Year
ORDER BY view_groups.GroupNum ASC;
$function$;


drop function if exists public.getfinalforminfo(pdisciplineid integer, pgroupid integer);
CREATE OR REPLACE FUNCTION public.getfinalforminfo(pdisciplineid integer, pgroupid integer)
  RETURNS TABLE("GroupNum" integer, "GroupName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying, "SpecCode" character varying, "FacultyID" integer, "FacultyName" character varying, "FacultyAbbr" character varying, "ExamType" exam_credit_grading_credit, "SubjectID" integer, "SubjectName" character varying, "SubjectAbbr" character varying, "AuthorID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "JobPosition" character varying, "Year" integer, "SemesterNum" integer, "StudyForm" character varying)
LANGUAGE sql
AS $function$
--select * from public.getfinalforminfo(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pgroupid 	-- put the pgroupid parameter value instead of 'pgroupid' (int4)
--);
SELECT  study_groups.GroupNum AS "GroupNum",
        groups_years.Name AS "GroupName",
        grades.ID AS "GradeID",
        grades.Num AS "GradeNum",
        grades.Degree AS "Degree",
        specializations.ID AS "SpecID",
        specializations.Name AS "SpecName",
        specializations.Abbr AS "SpecAbbr",
        specializations.Code AS "SpecCode",
        faculties.ID AS "FacultyID",
        faculties.Name AS "FacultyName",
        faculties.Abbr AS "FacultyAbbr",
        disciplines.ExamType AS "ExamType",
        subjects.ID AS "SubjectID",
        subjects.Name AS "SubjectName",
        subjects.Abbr AS "SubjectAbbr",
        teachers.ID AS "AuthorID",
        accounts.LastName AS "LastName",
        accounts.FirstName AS "FirstName",
        accounts.SecondName AS "SecondName",
        job_positions.Name AS "JobPosition",
  --departments.ID AS "DepID",
  --departments.Name AS "DepName",
        semesters."year" AS "Year",
        semesters.Num AS "SemesterNum",
        (SELECT study_form.FormName from study_form WHERE study_form.id = (
          select study_groups.formid from study_groups
          where study_groups.id = pgroupid
        )
        ) AS "StudyForm"
FROM study_groups
  INNER JOIN grades ON study_groups.GradeID = grades.ID
  INNER JOIN disciplines ON disciplines.ID = pDisciplineID
  INNER JOIN subjects ON disciplines.SubjectID = subjects.ID
  INNER JOIN teachers ON teachers.ID = disciplines.AuthorID
  INNER JOIN accounts ON teachers.AccountID = accounts.ID
  --INNER JOIN departments ON departments.ID = teachers.DepartmentID
  INNER JOIN job_positions ON job_positions.ID = teachers.JobPositionID
  INNER JOIN semesters ON disciplines.SemesterID = semesters.ID
  INNER JOIN groups_years ON groups_years.GroupID = study_groups.ID AND groups_years.Year = semesters.Year
  INNER JOIN specializations ON groups_years.SpecializationID = specializations.ID
  INNER JOIN faculties ON faculties.ID = specializations.FacultyID
WHERE study_groups.ID = pGroupID
LIMIT 1;
$function$;

DROP TYPE noukd_norate_full CASCADE;

drop function if exists public.discipline_getwithfullness(ptype noukd_norate_full, psemesterid integer, pfacultyid integer);

CREATE OR REPLACE FUNCTION public.DisciplineCollection_get(pSemesterID integer, pFacultyID integer)
  RETURNS TABLE("ID" integer, "SubjectName" VARCHAR(200), "AuthorID" integer, "State" TEXT)
LANGUAGE plpgsql
AS $function$
-- select public.discipline_getwithfullness(
--	:psemesterid,	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--	:pfacultyid 	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
-- );
BEGIN
  return query
  SELECT view_disciplines.DisciplineID AS ID,
         view_disciplines.SubjectName As SubjectName,
         view_disciplines.AuthorID As AuthorID,
         CASE
         WHEN (view_disciplines.MaxRate < 100 AND view_disciplines.IsLocked = 0) THEN 'no_map'
         WHEN (view_disciplines.MaxRate = 100 AND view_disciplines.IsLocked = 0) THEN 'no_rates'
         WHEN (view_disciplines.MaxRate = 100 AND view_disciplines.IsLocked = 1) THEN 'in_progress'
         ELSE 'other'
         END AS State
  FROM view_disciplines
  WHERE view_disciplines.SemesterID = pSemesterID AND
        view_disciplines.FacultyID = pFacultyID;
END
$function$;

DROP FUNCTION IF EXISTS public.teacher_create(plastname character varying, pfirstname character varying, psecondname character varying, pjobpositionid integer, pdepartmentid integer, pactivationcode character varying);
CREATE OR REPLACE FUNCTION public.teacher_create(plastname character varying, pfirstname character varying, psecondname character varying, pjobpositionid integer, pdepartmentid integer, pactivationcode character varying)
  RETURNS integer
LANGUAGE plpgsql
AS $function$

DECLARE vAccountID INT DEFAULT -1;
        vID integer;
begin
  --	select public.teacher_create(
  --	:plastname,	-- put the plastname parameter value instead of 'plastname' (varchar)
  --	:pfirstname,	-- put the pfirstname parameter value instead of 'pfirstname' (varchar)
  --	:psecondname,	-- put the psecondname parameter value instead of 'psecondname' (varchar)
  --	:pjobpositionid,	-- put the pjobpositionid parameter value instead of 'pjobpositionid' (int4)
  --	:pdepartmentid,	-- put the pdepartmentid parameter value instead of 'pdepartmentid' (int4)
  --	:pactivationcode 	-- put the pactivationcode parameter value instead of 'pactivationcode' (varchar)
  --);

  vAccountID := CreateAccount(pActivationCode, GetUserRole('teacher'));
  vAccountID := CreateAccount(pLastName, pFirstName, pSecondName, pActivationCode, GetUserRole('teacher'));
  -- add new teacher
  INSERT INTO teachers (AccountID, LastName, FirstName, SecondName, JobPositionID, DepartmentID) VALUES
    (vAccountID, pLastName, pFirstName, pSecondName, pJobPositionID, pDepartmentID) returning vID;
  INSERT INTO teachers (AccountID, JobPositionID, FacultyID) VALUES
    (vAccountID, pJobPositionID, pDepartmentID) returning ID into vID;
  RETURN vID;
  EXCEPTION
  when others then
    RETURN -1;

  --EXCEPTION
  --when others then
  --RETURN -1;

END;
$function$;

DROP FUNCTION IF EXISTS public.student_getrecordbooks(pstudentid integer);
CREATE OR REPLACE FUNCTION public.student_getrecordbooks(pstudentid integer)
  RETURNS TABLE("ID" integer, "ExternalID" character varying, "Degree" bachelor_master_specialist)
LANGUAGE sql
AS $function$
--	select  * from Student_GetRecordBooks(
--	:pStudentID 	-- put the pStudentID parameter value instead of 'pStudentID' (INT)
--);
select
  books."ID" as "ID",
  books."ExternalID" as "ExternalID",
  books."Degree" as "Degree"
from (
       SELECT DISTINCT
         view_students.RecordBookID       AS "ID",
         view_students.RecordBookNumber       AS "ExternalID",
         view_students."degree" as "Degree",
         view_students.SemesterId
       FROM view_students
       WHERE view_students.StudentID = pStudentID
       ORDER by view_students.SemesterId desc
     ) as books;

$function$;


CREATE SEQUENCE seq_logs_signin_detailed
  START WITH 1
  INCREMENT BY 1
  NO MINVALUE
  NO MAXVALUE
  CACHE 1;


CREATE TABLE logs_signin_detailed (
  id integer DEFAULT nextval('seq_logs_signin_detailed'::regclass) NOT NULL,
  accountid integer,
  globalkey character varying NOT NULL,
  isstudent integer,
  isstaff integer,
  error_message character varying,
  date timestamp(0) without time zone DEFAULT now() NOT NULL
);

CREATE INDEX key_logs_signin_detailed ON logs_signin_detailed USING btree (globalkey);

ALTER TABLE ONLY logs_signin_detailed
  ADD CONSTRAINT logs_signin_detailed_pkey PRIMARY KEY (id);

CREATE OR REPLACE FUNCTION public.log_sigindetailed(paccountid integer, pisstaff integer, pisstudent integer, pglobalkey character varying, pmessage character varying)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
begin
  INSERT INTO logs_signin_detailed (AccountID, GlobalKey, IsStudent, IsStaff, error_message) VALUES (paccountid, pglobalkey, pisstudent, pisstaff, pmessage);
  RETURN 0;
END
$function$;
